/*
Code by Marcin Ignac 2010
http://marcinignac.com
*/

import traer.physics.*;
import processing.opengl.*;

ParticleSystem physics;
Particle[][] particles;
int gridSize = 20;

float SPRING_STRENGTH = 0.2;
float SPRING_DAMPING = 0.1;

float gridStepX = 0.75;
  float gridStepY = 0.75;

Camera cam;
boolean enabled = false;

void setup() {
  size(1280/2, 720/2, OPENGL);
  hint(ENABLE_OPENGL_4X_SMOOTH);
  hint(DISABLE_OPENGL_2X_SMOOTH);
  noSmooth();
  fill(0);
  
  cam = new Camera();  
        
  physics = new ParticleSystem(-0.001, 0.01);
  
  particles = new Particle[gridSize][gridSize];
  
  for (int i = 0; i < gridSize; i++) {
    for (int j = 0; j < gridSize; j++) {
      particles[i][j] = physics.makeParticle(0.2, j * gridStepX - gridSize * gridStepX / 2, 0.0, i * gridStepY - gridSize * gridStepY / 2);
      if (j > 0) {
        physics.makeSpring(particles[i][j - 1], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepX);
      }
    }
  }
  
  for (int j = 0; j < gridSize; j++) {
    for (int i = 1; i < gridSize; i++) {
      physics.makeSpring(particles[i - 1][j], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepY);
    }
  }
  
  particles[0][0].makeFixed();
  particles[0][gridSize - 1].makeFixed();
  particles[gridSize - 1][gridSize - 1].makeFixed();
  particles[gridSize - 1][0].makeFixed();
  
  particles[5][5].makeFixed();
  particles[6][5].makeFixed();
  particles[7][5].makeFixed();
  particles[7][6].makeFixed();
  particles[6][6].makeFixed();
  particles[6][7].makeFixed();
  particles[5][7].makeFixed();
  particles[5][6].makeFixed();
  /*
  particles[13][13].makeFixed();
  particles[12][13].makeFixed();
  particles[11][13].makeFixed();
  particles[11][14].makeFixed();
  particles[12][14].makeFixed();
  particles[12][15].makeFixed();
  particles[13][15].makeFixed();
  particles[13][14].makeFixed();
  */
}

void draw()
{
  if (enabled) physics.tick();
  
  if (mousePressed)
  {
    enabled = true;
    for (int i = 0; i < gridSize; i++) {
      for (int j = 0; j < gridSize; j++) {
        if (i > 0 && j > 0 && i < gridSize - 1 && j < gridSize - 1) {
        if (particles[i][j].isFixed()) {
            particles[i][j].position().setX( particles[i][j].position().x() * 0.99);
            particles[i][j].position().setZ( particles[i][j].position().z() * 0.99);
        }
        }
      }
     }
  }
  
  noFill();
  
  background(255);
  
  cam.perspective(60, 1280.0/720.0, 0.1, 1000);  
  cam.lookAt(new Vec3(13*cos(frameCount/100.0), 8, 13*sin(frameCount/100.0)), new Vec3(0, -4, 0), new Vec3(0, 1, 0));
  GL gl = ((PGraphicsOpenGL)g).gl; 
  
  gl.glBegin(GL.GL_QUADS);
  gl.glColor3f(1,1,0);
  for (int i = 1; i < gridSize; i++) {
     for (int j = 1; j < gridSize; j++) {
      float y = -particles[i][j].position().y()/5.0;
      gl.glColor3f(1, 1-y, 0);
      gl.glVertex3f(particles[i][j].position().x(), particles[i][j].position().y(), particles[i][j].position().z());
      gl.glVertex3f(particles[i-1][j].position().x(), particles[i-1][j].position().y(), particles[i-1][j].position().z());
      gl.glVertex3f(particles[i-1][j-1].position().x(), particles[i-1][j-1].position().y(), particles[i-1][j-1].position().z());
      gl.glVertex3f(particles[i][j-1].position().x(), particles[i][j-1].position().y(), particles[i][j-1].position().z());
    }
  }
  gl.glEnd();
  
  gl.glTranslatef(0, 0.01, 0);
  for (int i = 0; i < gridSize; i++)
  {
    gl.glBegin(GL.GL_LINES);
    gl.glColor3f(0,0,0);
    for (int j = 1; j < gridSize; j++) {
      gl.glVertex3f(particles[i][j-1].position().x(), particles[i][j-1].position().y(), particles[i][j-1].position().z());
      gl.glVertex3f(particles[i][j].position().x(), particles[i][j].position().y(), particles[i][j].position().z());
    }
 
    gl.glEnd();
  }
  for (int j = 0; j < gridSize; j++)
  {
    gl.glBegin(GL.GL_LINES);
    gl.glColor3f(0,0,0);
    for (int i = 1; i < gridSize; i++) {
      gl.glVertex3f(particles[i-1][j].position().x(), particles[i-1][j].position().y(), particles[i-1][j].position().z());
      gl.glVertex3f(particles[i][j].position().x(), particles[i][j].position().y(), particles[i][j].position().z());
    }
    
    gl.glEnd();
  }
}

void mouseReleased()
{
  for (int i = 0; i < gridSize; i++) {
      for (int j = 0; j < gridSize; j++) {
        particles[i][j].position().setX(i * gridStepX - gridSize * gridStepX / 2);
        particles[i][j].position().setY(0);
        particles[i][j].position().setZ(j * gridStepX - gridSize * gridStepX / 2);
      }
     }
}
